@@ -59,5 +59,13 @@ class ABCLabelProcessor(ABC):
                           threshold= 0.5) :
         raise NotImplementedError
 
+    def smooth_labels(self,labels, factor=0.1):
+        # smooth the labels
+        labels *= (1 - factor)
+        labels += (factor / labels.shape[1])
+
+        # returned the smoothed labels
+        return labels
+
 
 
