@@ -45,13 +45,21 @@ class SpanClassifyModel(ABCLabelingModel):
         ]
 
         tensor = embed_model.output
-        for layer in layer_stack:
-            tensor = layer(tensor)
-
+        tensor = L.Dropout(**config['layer_dropout'])(tensor)
         tensor = self.mask_save_layer(tensor)
 
+
+        tensor = tf.keras.layers.Dense(128)(tensor)
+        end_node = tf.keras.layers.Dense(128)(tensor)
+        tensor = self.refomer_layer(tensor,128)
+        end_node = self.refomer_layer(end_node,128)
+        # for layer in layer_stack:
+        #     tensor = layer(tensor)
+
+
+
         starts_logits =  layers_starts_label_fc(tensor)
-        ends_logits = layers_ends_label_fc([tensor,starts_logits])
+        ends_logits = layers_ends_label_fc([end_node,starts_logits])
 
         output_starts = tf.nn.softmax(starts_logits)
         output_ends = tf.nn.softmax(ends_logits)
@@ -71,7 +79,9 @@ class SpanClassifyModel(ABCLabelingModel):
                       **kwargs: Any) -> None:
 
         if loss is None:
-            self.mask_save_layer.set_loss_func(tf.keras.losses.sparse_categorical_crossentropy)
+            from tensorflow_addons.losses import SigmoidFocalCrossEntropy
+            from tensorflow.keras.losses import BinaryCrossentropy
+            self.mask_save_layer.set_loss_func(SigmoidFocalCrossEntropy())
             loss = self.mask_save_layer.loss
         if metrics is None:
             metrics = 'accuracy'
@@ -112,3 +122,53 @@ class SpanClassifyModel(ABCLabelingModel):
         res = self.label_processor.inverse_transform((starts_labels.tolist(),ends_labels.tolist()),lengths=lengths,mapping_list=x_data_mapping)
         return res
 
+
+
+    def refomer_layer(self,inputs,hidden_size):
+        from bert4keras.layers import SinusoidalPositionEmbedding
+        from tensorflow.keras import backend as K
+        pos = SinusoidalPositionEmbedding(hidden_size, 'zero')(inputs)
+
+        cos_pos = K.repeat_elements(pos[..., 1::2], 2, -1)
+        sin_pos = K.repeat_elements(pos[..., ::2], 2, -1)
+        inputs2 = K.stack([-inputs[..., 1::2], inputs[..., ::2]], 3)
+        inputs2 = K.reshape(inputs2, K.shape(inputs))
+        new_input = inputs * cos_pos + inputs2 * sin_pos
+        return new_input
+
+    def gloablPointerLayer(self,inputs,mask,hidde_size=128):
+        from bert4keras.layers import SinusoidalPositionEmbedding
+        from tensorflow.keras import backend as K
+        from nlp_tools.utils.keras_utils import sequence_masking
+
+
+        inputs = tf.keras.layers.Dense(hidde_size *2)(inputs)
+        pos = SinusoidalPositionEmbedding(hidde_size, 'zero')(inputs)
+        input_start,input_end = inputs[...,:hidde_size],inputs[...,hidde_size:]
+
+        cos_pos = K.repeat_elements(pos[..., 1::2], 2, -1)
+        sin_pos = K.repeat_elements(pos[..., ::2], 2, -1)
+        input_start2 = K.stack([-input_start[..., 1::2], input_start[..., ::2]], 3)
+        input_start2 = K.reshape(input_start2, K.shape(input_start))
+        new_input_start = input_start * cos_pos + input_start2 * sin_pos
+
+        input_end2 = K.stack([-input_end[..., 1::2], input_end[..., ::2]], 3)
+        input_end2 = K.reshape(input_end2, K.shape(input_end))
+        new_input_end = input_end * cos_pos + input_end2 * sin_pos
+
+        # 计算内积
+        logits = tf.einsum('bmhd,bnhd->bhmn', new_input_start, new_input_end)
+
+        # 排除padding
+        logits = sequence_masking(logits, mask, '-inf', 2)
+        logits = sequence_masking(logits, mask, '-inf', 3)
+
+        # 排除下三角
+        mask = tf.linalg.band_part(K.ones_like(logits), 0, -1)
+        logits = logits - (1 - mask) * 1e12
+
+        # scale返回
+        logits =  logits / hidde_size ** 0.5
+
+
+        return new_input_start,new_input_end,logits
\ No newline at end of file
