@@ -87,4 +87,44 @@ def output_ner_results(origin_texts,predict_labels):
     return result
 
 
+def span_extract_item(start_logits, end_pred):
+    '''return type list
+    [(labeltype，start,end)]
+    '''
+    S = []
+    for i, s_l in enumerate(start_logits):
+        if s_l == 0:
+            continue
+        for j, e_l in enumerate(end_pred[i:]):
+            if s_l == e_l:
+                S.append((s_l, i, i + j))
+                break
+    return S
+
+
+def span_extract_item_no_hu(start_logits, end_pred):
+    '''return type list
+    [(labeltype，start,end)]
+    '''
+    S = []
+    for i, s_l in enumerate(start_logits):
+        if s_l == 0:
+            continue
+        for j, e_l in enumerate(end_pred[i:]):
+            if s_l == e_l:
+                S.append([s_l, i, i + j])
+                break
+
+    sorted_S = sorted(S,key=lambda x:x[1])
+
+    output = []
+    for before,after in zip(sorted_S[:-1],sorted_S[1:]):
+        if before[2] >= after[1]:
+            before[2] = after[1] -1
+        output.append(before)
+    if sorted_S:
+        output.append(sorted_S[-1])
+
+    return output
+
 
